#!/usr/bin/perl
use lib "blib/lib";

#
# Monolithic EXE and PAR distributions generator script: substitutes
# for both a BAT and an UNIX shell script, and for the tar utility as
# well (Archive::Tar is a core module).
#
# Antonio García Domínguez
# $Id$
#

use strict;
use warnings;
use English;
use Config;
use Fatal;

use Archive::Tar;
use File::Temp qw(tempdir);
use File::Copy qw(copy);
use File::Spec;
use File::Spec::Functions;
use YAXML::Reverse;


### CONFIGURATION

use constant {
    INCLUDE_DIRS => ["xslt"],
    MAIN_SCRIPT  => "script/yaml2xml",
    MODULE_NAME  => "YAXML-Reverse",
    BUILD_ARCHSPECIFIC_PAR => 1,
    ONLY_COPY    => qr/^[^.].*/,
    TYPE_FILE    => "types/yaml.type",
    VERSION      => $YAXML::Reverse::VERSION,
};

### MAIN BODY

# List all files of interest for the distributions
my @common_files = recursive_filtered_list('filter' => ONLY_COPY,
					   'rv_list_queue' => INCLUDE_DIRS);

# EXE generation
my $path_to_exe = (File::Spec->splitpath(MAIN_SCRIPT))[2];
my $exe_command;
if ($OSNAME =~ /^(?:MSWin|cygwin|OS2)/) {
    $path_to_exe .= ".exe";
    $exe_command  = "$path_to_exe %s";
}
else {
    $exe_command  = "./$path_to_exe %s";
}
system("pp -r -I blib/lib -M PerlIO " 
       . " -o $path_to_exe "
       . MAIN_SCRIPT);

# PAR generation
my $path_to_par = (File::Spec->splitpath(MAIN_SCRIPT))[2] . ".par";
my $par_command = "perl -MPAR $path_to_par %s";
system("pp -r -I blib/lib -M PerlIO -p" 
       . " -o $path_to_par "
       . MAIN_SCRIPT);

# Variants to be packaged
my @variants = (
    {'extra_files' => [$path_to_exe],
     'command'     => $exe_command,
     'name'        => 
	 MODULE_NAME 
	 . "-" . VERSION . "-standalone-$Config{archname}"},
    {'extra_files' => [$path_to_par],
     'command'     => $par_command,
     'name'        => 
	 MODULE_NAME 
	 . "-" . VERSION . "-par-" 
	 . (BUILD_ARCHSPECIFIC_PAR ? $Config{archname} : "noarch")}
    );

foreach my $variant (@variants) {
    # Add common and variant-specific files
    my $tar       = Archive::Tar->new;
    $tar->add_files(@common_files, @{$variant->{extra_files}});

    # Add a modified version of the format descriptor
    my $command = $variant->{command};

    local *TYPE_FH;
    open(TYPE_FH, "<", TYPE_FILE)
	or die "Couldn't open " . TYPE_FILE . ": $!";
    my @lines = <TYPE_FH>;
    close(TYPE_FH);

    @lines    = map { /import_cmd/ && "<import_cmd>$command</import_cmd>" || $_ } @lines;
    $tar->add_data(TYPE_FILE, join("\n", @lines));

    # Write the tar.gz to disk
    $tar->write($variant->{name} . ".tar.gz", 1);
}

### UTILITY FUNCTIONS

sub recursive_filtered_list
{
    my %args = @_;

    my $filter_pattern = $args{filter};
    my @list_queue     = @{$args{rv_list_queue}};
    my @results        = ();

    while (@list_queue > 0) 
    {
	# Relative path from the current directory
	my $path = shift @list_queue;

	if (-d $path) {
	    my $olddir;
	    opendir($olddir, $path)
		or die "Couldn't open $path: $!";
	    my @entries = readdir($olddir);
	    closedir($olddir);

	    # Add filtered entries to queue
	    @entries = grep /$filter_pattern/, @entries;
	    @entries = map {catfile($path, $_);} @entries;
	    push @list_queue, @entries;
	}

	if (-f $path) {
	    push @results, $path;
	}
    }

    return @results;
}
